@@ -1,6 +1,6 @@
 package Text::Modify;
 #================================================================
-# (C)2004-2005, lammel@cpan.org
+# (C)2004, rl AT brabbel.net
 #================================================================
 # - Multiline replace is NOT supported currently
 # - only simple regex and string replacement probably works the
@@ -15,7 +15,7 @@ use Text::Buffer;
 use vars qw($VERSION);
 
 BEGIN {
-	$VERSION="0.4.1";
+	$VERSION="0.2";
 }
 
 sub new {
@@ -43,9 +43,8 @@ sub new {
     	if ($self->{writeto}) { $self->{backup} = 0; }
     }
     else { $self->{file} = shift; }
-    if (!$self->{writeto} && $self->{file}) { $self->{writeto} = $self->{file}}
     $self->_debug("Created object $class as $self (" . ref($self) . ")");
-    $self->_clearError();
+    $self->clearError();
     $self->{ruleorder} = [];
     $self->{blockorder} = [];
     # Define the "ALL" block, which includes the whole file and is used
@@ -54,61 +53,60 @@ sub new {
     return $self;
 }
 
-# Block processing not implemented yet
-#sub defineBlock {
-#	my $self = shift;
-#	my $name = shift;
-#	my %opts = @_;
-#	if (exists($self->{block}->{$name})) {
-#		$self->_setError("Block $name already defined");
-#		return 0;
-#	}
-#	if ($opts{fromline}) {
-#		$self->{block}->{$name}->{from} = $opts{fromline};
-#	} elsif ($opts{frommatch}) {
-#		$self->{block}->{$name}->{frommatch} = $opts{frommatch};
-#	} else {
-#		$self->{block}->{$name}->{from} = 0;
-#	}
-#	if ($opts{toline}) {
-#		$self->{block}->{$name}->{to} = $opts{toline};
-#	} elsif ($opts{frommatch}) {
-#		$self->{block}->{$name}->{tomatch} = $opts{tomatch};
-#	} else {
-#		$self->{block}->{$name}->{to} = 999999;
-#	}
-#	push @{$self->{blockorder}},$name;
-#	return 1;
-#}
-#
-#sub undefineBlock {
-#	my $self = shift;
-#	my $name = shift;
-#	if (exists($self->{block}->{$name})) {
-#		$self->_debug("Undefining block $name");
-#		delete($self->{block}->{$name});
-#		my @tmp = @{$self->{blockorder}};
-#		@{$self->{blockorder}} = grep($_ ne $name, @tmp);    
-#	} else {
-#		$self->_debug("Block $name not defined, ignoring");
-#	}
-#	return 1;
-#}
-#
-#sub listMatchBlocks {
-#	my $self = shift;
-#	return (grep { !defined($self->{block}->{$_}->{from}) || !defined($self->{block}->{$_}->{to}) } $self->listBlocks());
-#}
-#
-#sub listCurrentBlocks {
-#	my $self = shift;
-#	return (grep { $self->{block}->{$_}->{active} } $self->listBlocks());
-#}
-#
-#sub listBlocks {
-#	my $self = shift;
-#	return @{$self->{blockorder}};
-#}
+sub defineBlock {
+	my $self = shift;
+	my $name = shift;
+	my %opts = @_;
+	if (exists($self->{block}->{$name})) {
+		$self->setError("Block $name already defined");
+		return 0;
+	}
+	if ($opts{fromline}) {
+		$self->{block}->{$name}->{from} = $opts{fromline};
+	} elsif ($opts{frommatch}) {
+		$self->{block}->{$name}->{frommatch} = $opts{frommatch};
+	} else {
+		$self->{block}->{$name}->{from} = 0;
+	}
+	if ($opts{toline}) {
+		$self->{block}->{$name}->{to} = $opts{toline};
+	} elsif ($opts{frommatch}) {
+		$self->{block}->{$name}->{tomatch} = $opts{tomatch};
+	} else {
+		$self->{block}->{$name}->{to} = 999999;
+	}
+	push @{$self->{blockorder}},$name;
+	return 1;
+}
+
+sub undefineBlock {
+	my $self = shift;
+	my $name = shift;
+	if (exists($self->{block}->{$name})) {
+		$self->_debug("Undefining block $name");
+		delete($self->{block}->{$name});
+		my @tmp = @{$self->{blockorder}};
+		@{$self->{blockorder}} = grep($_ ne $name, @tmp);    
+	} else {
+		$self->_debug("Block $name not defined, ignoring");
+	}
+	return 1;
+}
+
+sub listMatchBlocks {
+	my $self = shift;
+	return (grep { !defined($self->{block}->{$_}->{from}) || !defined($self->{block}->{$_}->{to}) } $self->listBlocks());
+}
+
+sub listCurrentBlocks {
+	my $self = shift;
+	return (grep { $self->{block}->{$_}->{active} } $self->listBlocks());
+}
+
+sub listBlocks {
+	my $self = shift;
+	return @{$self->{blockorder}};
+}
 
 ### TODO Need to define all methods and also options like
 ### TODO addIfMissing to add a required line even if it is not found at end/start of file or block
@@ -123,15 +121,14 @@ sub defineRule {
 	if (!$name) {
 		$name = "rule" . ($#{$self->{ruleorder}}+1);
 	}
-	return 0 if (!%opts);
 	$self->_debug("Defining rule '$name': " . join(",",%opts));
 	if (!$opts{replace} && !$opts{insert} && !$opts{'delete'}) {
-		$self->_addError("Failed to define rule $name");
+		$self->addError("Failed to define rule $name");
 		return 0;
 	}
 	$self->{rule}->{$name} = new Text::Modify::Rule(%opts, debug => $self->{_debug});
 	if (!$self->{rule}->{$name}) {
-		$self->_setError("Could not init rule $name");
+		$self->setError("Could not init rule $name");
 		return 0;
 	}
 	push @{$self->{ruleorder}},$name;
@@ -152,36 +149,20 @@ sub undefineRule {
 	return 1;
 }
 
-# Simple syntax ->replace("MY","HIS") or ->replaceLine("WHAT","WITH",ignorecase => 1) 
+# Simple syntax ->replaceLine("MY","HIS") or ->replaceLine("WHAT","WITH",ignorecase => 1) 
 # supported options are: 
 # 	dryrun		do not apply changes
 #	ignorecase	ignore case for matching
 #	ifmissing 	insert/append/ignore/fail string if missing (cannot use results of regex then)
 # 	matchfirst	only match X times for replacing, 1 would only replace the first occurence
-sub replace {
-	my $self = shift;
-	return $self->replaceRegex(@_);
-}
-
-sub replaceString {
+sub replaceLine {
 	my ($self,$what,$with,%opts) = @_;
-	$self->_debug("Adding string replace rule: '$what' with '$with'");
-	return $self->defineRule(replace=>$what,type=>'string',string=>$what,with=>$with,%opts);
+	$opts{replace} = $what;
+	$opts{with} = $with;
+	return $self->defineRule(%opts);
 }
 
-sub replaceWildcard {
-	my ($self,$what,$with,%opts) = @_;
-	$self->_debug("Adding wildcard replace rule: '$what' with '$with'");
-	return $self->defineRule(replace=>$what,type=>'wildcard',wildcard=>$what,with=>$with,%opts);
-}
-
-sub replaceRegex {
-	my ($self,$what,$with,%opts) = @_;
-	$self->_debug("Adding regex replace rule: '$what' with '$with'");
-	return $self->defineRule(replace=>$what,type=>'regex',regex=>$what,with=>$with,%opts);
-}
-
-# TODO sub replaceInBlock { }
+sub replaceInBlock { }
 
 # Usage: Delete line matching expressions MATCH
 # Syntax: ->deleteLine("MATCH", ignorecase => 1, matchfirst => 1)
@@ -191,29 +172,28 @@ sub replaceRegex {
 #	ifmissing 	ignore|fail if missing
 # 	matchfirst	only match X times for replacing, 1 would only replace the first occurence
 
-sub delete { 
+sub deleteLine { 
 	my ($self,$what,%opts) = @_;
 	$opts{'delete'} = $what;
 	return $self->defineRule(%opts);
 }
-# TODO sub deleteInBlock { }
+sub deleteInBlock { }
 
-sub insert { 
+sub insertLine { 
 	my ($self,$what,%opts) = @_;
 	$opts{insert} = $what;
 	$opts{at} = "top";
 	return $self->defineRule(%opts);	
 }
-# TODO sub insertInBlock { }
+sub insertLineInBlock { }
 
-sub append { 
+sub appendLine { 
 	my ($self,$what,%opts) = @_;
 	$opts{insert} = $what;
 	$opts{at} = "bottom";
 	return $self->defineRule(%opts);	
 }
-
-# TODO sub appendInBlock { }
+sub appendLineInBlock { }
 
 
 sub listRules {
@@ -233,7 +213,7 @@ sub backupExtension {
 	return $self->{backupExt};
 }
 
-sub _getBackupFilename {
+sub getBackupFilename {
 	my $self = shift;
 	my $file = $self->{'file'} || shift;
 	my $bakfile = $file . $self->{'backupExt'};
@@ -256,7 +236,7 @@ sub _getBackupFilename {
 sub createBackup {
 	my $self = shift;
 	my $file = $self->{'file'} || shift;
-	my $bakfile = $self->_getBackupFilename();
+	my $bakfile = $self->getBackupFilename();
 	### Create a backup if bakfile is set
 	if ($bakfile && $bakfile ne $file) {
 		$self->_debug("- Creating backup copy $bakfile");
@@ -266,6 +246,91 @@ sub createBackup {
 	return $bakfile;
 }
 
+
+
+
+sub _processLine {
+	my $self = shift;
+	my $line = shift;
+	# TODO Do the actual processing, block detection and replacing
+	my $changed = 0;
+	my $out = $line;
+	foreach my $rulekey (keys %{$self->{'rule'}}) {
+		my $string = undef;
+		my $rule = $self->{'rule'}->{$rulekey};
+		$self->_debug("Processing rule $rulekey (" . ref($rule) . ")");
+		if ($rule) {
+			$string = $rule->process($out);
+		}
+		$self->_debug("Line before(<<<)/after(>>>):\n<<<$line>>>" . ($string || "undef"));
+		if (defined($string)) {
+			$self->{replacecount} += $rule->getReplaceCount();
+			$self->{matchcount}   += $rule->getMatchCount();
+			if ($string ne $line) {
+				$changed = 1;
+				$out = $string;
+			}
+		}
+	
+	}
+	if ($changed) { $self->{lineschanged}++ }
+	if (!$self->{dryrun}) {
+		if ($self->{tmpfh}->opened()) {
+			$self->{tmpfh}->print($out);
+		} else {
+			$self->setError("Cannot write to temp. file $self->{tmpname}");
+			return 0;
+		}
+	}
+	return $line;
+}
+
+sub _readFile {
+	my $self = shift;
+	my $file = $self->{'file'};
+	$self->{linesread} = 0;	
+	if (-r $file && open(IN,$file)) {
+		while (<IN>) {
+			# TODO Actually process the line and do the replacing
+			$self->{linesread}++;			
+			$self->_debug("Read line " . $self->{linesread} . ": $_");
+			push @{$self->{data}},$_;
+		}
+		close(IN);
+		if ($self->isError()) {
+			Error($self->getError());
+			return 0;
+		}
+	} else {
+		$self->setError("Cannot read from $file");
+		return 0;
+	}
+	return 1;
+}
+
+sub _writeFile {
+	my $self = shift;
+	my $file = $self->{'writeto'};
+	$self->{lineswritten} = 0;
+	# TODO maybe should move back processing here	
+	if (-w $file && open(OUT,">$file")) {
+		$self->_debug(3,"Writing to file $file");
+		foreach (@{$self->{data}}) {
+			$self->_debug("Writing: $_");
+			print OUT $_;
+			$self->{lineswritten}++;
+		}
+		if (!close(OUT)) {
+			$self->setError("Failed to close file after $self->{lineswritten} lines written");
+			return 0;
+		}
+	} else {
+		$self->setError("Cannot write file $file");
+		return 0;
+	}
+	return 1;
+}
+
 sub process {
 	my $self = shift;
 	my $file = $self->{'file'};
@@ -283,6 +348,12 @@ sub process {
 	$self->{_buffer} = $txtbuf;
 	$self->_debug("Read $self->{linesread} from $file");
 
+# TODO Need this for processing of large files	
+#	if (!$self->{writeto}) {
+#		($self->{tmpfh},$self->{tmpname}) = tempfile();
+#		$self->_debug(2,"Using temp. file: $self->{tmpname}");
+#	}
+
 	$self->{replacecount} = 0;
 	$self->{matchcount} = 0;
 	$self->{addcount} = 0;
@@ -294,17 +365,15 @@ sub process {
 	foreach ($self->listRules()) {
 		my $rule = $self->{rule}->{$_};
 		$self->_debug("Processing rule $_");
-		my $changecount = $rule->process($self->{_buffer});
-		$self->{changecount} += $changecount;
-		my ($match, $add, $del, $repl) = $rule->getModificationStats();
-		$self->{replacecount} += $repl;
-		$self->{matchcount} += $match;
-		$self->{addcount} += $add;
-		$self->{deletecount} += $del;
+		$rule->process($self->{_buffer});
+		$self->{replacecount} += $rule->getReplaceCount();
+		$self->{matchcount} += $rule->getMatchCount();
+		$self->{addcount} += $rule->getAddCount();
+		$self->{deletecount} += $rule->getDeleteCount();
 		$self->_debug("Stats rule $_ (change/match/repl/add/del): " . 
-			"$self->{lineschanged}/$match/$repl/$add/$del");
+			"$self->{lineschanged}/$self->{matchcount}/$self->{replacecount}/$self->{addcount}/$self->{deletecount}");
 		if ($rule->isError()) {
-			$self->_addError($rule->getError());
+			$self->addError($rule->getError());
 			last;
 		}
 	}
@@ -314,14 +383,11 @@ sub process {
 	
 	### Now mv the temp. file to overwrite the original configfile
 	if (!$self->{dryrun}) {
-		# Force saving now
-		$self->{_buffer}->setModified();
 		if (!$self->{_buffer}->save($self->{writeto})) {
-			$self->_debug("Error saving file to " . $self->{writeto});
 			return 0;
 		}
 	} else {
-		$self->_debug("Dryrun, not writing file");
+		$self->_debug("Dryrun, not writing file")
 	}
 	$self->_debug("Statistics:
 	Lines read: 	$self->{linesread}   
@@ -335,10 +401,9 @@ sub process {
 
 sub dryrun {
 	my $self = shift;
-	my $old = $self->{dryrun};
 	$self->{dryrun} = 1;
 	my $rc = $self->process();
-	$self->{dryrun} = $old;
+	$self->{dryrun} = 0;
 	return $rc;
 }
 
@@ -350,20 +415,19 @@ sub getMatchCount     { return shift->{matchcount}; }
 sub getAddCount       { return shift->{addcount}; }
 sub getDeleteCount    { return shift->{deletecount}; }
 
-
 #=============================================================
 # ErrorHandling Methods
 #=============================================================
-sub _addError { my $self = shift; $self->{error} .= shift; }
+sub addError { my $self = shift; $self->{error} .= shift; }
 sub isError { return (shift->{'error'} ? 1 : 0); }
-sub _setError { my $self = shift; $self->{error} = shift; }
+sub setError { my $self = shift; $self->{error} = shift; }
 sub getError {
 	my $self = shift;
 	my $error = $self->{error};
-	$self->_clearError();
+	$self->clearError();
 	return $error;
 }
-sub _clearError { shift->{error} = ""; }
+sub clearError { shift->{error} = ""; }
 
 #=============================================================
 # Private methods (for internal use )
@@ -372,188 +436,9 @@ sub _clearError { shift->{error} = ""; }
 # Only internal function for debug output
 sub _debug {
 	my $self = shift;
-	if ($#_ == -1) {
-		return $self->{_debug};
-	}
-	elsif ( $self->{_debug} ) {
-		print "[DEBUG] @_\n";
+	if ($self->{_debug}) {
+		print "@_\n";
 	}
 }
 
-1;
-__END__
-
-=head1 NAME
-
-Text::Modify - oo-style interface for simple, rule-based text modification
-
-=head1 SYNOPSIS
-
-  use Text::Modify;
-
-  my $mod = new Text::Modify(-file=>'my.txt', -writeto=>'new.txt', dryrun=>0);
-  $mod->replace("sad","funny");
-  $mod->replace('.*?logalhos$',"127.0.0.1	localhost",ifmissing=>'append');
-  my $count = $mod->process();
-
-=head1 DESCRIPTION
-
-C<Text::Modify> is a simple oo-style interface to perform variuos 
-text modifcation tasks.
-
-Instead of having to parse and modify textfiles with customized
-routines over and over C<Text::Modify> provides a common ruleset, 
-which allows simple to advanced editing tasks to be performend.
-
-After instantiating a new C<Text::Modify> object, rules are defined
-on it and finally processed.
-
-	my $mod = new Text::Modify();
-
-C<Text::Modify> uses C<Text::Buffer> internally to perform the 
-editing tasks on the text-file.
-
-=head1 Methods
-
-=over 8
-
-=item new
-
-    $mod = new Text::Modify(%options);
-
-This creates a new object, starting with an empty buffer unless the
-B<-file> or B<-array> options are provided. The available
-attributes are:
-
-=item append
-
-	$mod->append("new last line");
-
-Add a rule to append a new line at enf of text.	
-	
-=item insert
-
-	$mod->insert("new first line");
-
-Add a rule to insert a new line at start of text.	
-
-=item delete
-
-	$mod->delete('.*DELETE ME$');
-
-Add a rule to delete lines matching the supplied string. The string is
-interpreted as a regular expression, so be sure to escape characters
-accordingly.
-
-=item replace
-
-	$mod->replace("foo","bar", ifmissing=>append, ignorecase=>1);
-
-Add a rule to replace all occurences of C<foo> with C<bar> in the text.	
-
-=item replaceString
-
-	$mod->replaceString("foo","bar", ifmissing=>append, ignorecase=>1);
-
-Add a rule to replace all occurences of string C<foo> with C<bar> in the text.	
-
-=item replaceWildcard
-
-	$mod->replace("*foo?","bar", ifmissing=>append, ignorecase=>1);
-
-Add a rule to replace all occurences matching the wildcard C<*foo?> with
-C<bar> in the text.	'*' (asterisk) will match any characters (as much as
-possible) and '?' (question mark) will match one character
-
-=item replaceRegex
-
-	$mod->replace("\s*foo\d+","bar", ifmissing=>append);
-	$mod->replace("\s*foo(\d+)",'bar$1', ignorecase=>1);
-
-Add a rule to replace all occurences matching the regular expression 
-C<*foo?> with C<bar> in the text. Also regex parameters can be used in
-the replacement string.
-
-=item defineRule
-
-	$mod->defineRule(replace=>'foo\s+bar',with=>'foobar', ifmissing=>append);
-
-# TODO add pod for all options supported by defineRule()
-Advanced interface to define a rule, which gives most flexibilty
-to perform a given task the way you need it.
-
-=item undefineRule
-
-Delete a rule, that was created with the supplied name
-
-=item listRules
-
-Returns a list of rules in the order they will be executed.
-
-=item createBackup
-
-create a backup of the specified file
-
-=item backupExtension
-
-get/set the backup extension used for backup files
-
-=item getLinesModified
-=item getLinesProcessed
-=item getDeleteCount
-=item getAddCount
-=item getMatchCount
-=item getReplaceCount
-
-Return statistics and counters of the processing performed
-
-=item process
-
-	$mod->process();
-
-Start processing of the text, rule by rule. If dryrun is enabled, 
-modification will be performed in memory, but are B<not> written
-to file.
-
-=item dryrun
-
-	$mod->dryrun();
-
-Start processing of the text, rule by rule with dryrun enabled. 
-The setting for dryrun will be restored after processing.
-Modification will be performed in memory, but are B<not> written
-to file.
-
-=item isDryRun
-
-Returns 1 if dryrun has been enabled, no modifications will be written
-to the text to process. Otherwise returns 0.
-
-=item isError
-
-=item getError
-
-	if ($text->isError()) { print "Error: " . $text->getError() . "\n"; }
-
-Simple error handling routines. B<isError> returns 1 if an internal error
-has been raised. B<getError> returns the textual error.
-
-=back
-
-=head1 BUGS
-
-There definitly are some, if you find some, please report them.
-
-=head1 LICENSE
-
-This software is released under the same terms as perl itself. 
-You may find a copy of the GPL and the Artistic license at 
-
-   http://www.fsf.org/copyleft/gpl.html
-   http://www.perl.com/pub/a/language/misc/Artistic.html
-
-=head1 AUTHOR
-
-Roland Lammel (lammel@cpan.org)
-
-=cut
+1;
\ No newline at end of file
